package org.openxma.dsl.generator.helper;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.EcoreUtil2;
import org.openxma.dsl.core.model.FalseLiteral;
import org.openxma.dsl.pom.model.ComposeData;
import org.openxma.dsl.pom.model.Content;
import org.openxma.dsl.pom.model.Page;
import org.openxma.dsl.pom.model.SetOfGuiElements;
import org.openxma.dsl.pom.model.StyleSpecification;

/**
 * Handling of propagation of styles
 *
 */
public class StylePropertyPropagation {
    
    static boolean isTypeWithAutomaticPropagation(EObject obj) {
        if (obj instanceof SetOfGuiElements) {
            return true;           
        }
        if (obj instanceof ComposeData) {
            return true;  
        }
        if (obj instanceof Content) {
            return true;
        }
        if (obj instanceof Page) {
            return true;
        }                                
        return false;
    }
    
    static boolean isPropagated(StyleSpecification styleSpecification, EObject obj) {      
        if ((styleSpecification.getPropagation() instanceof FalseLiteral == false) || isTypeWithAutomaticPropagation(obj)) {
            return true;
        }
        return false;
    }
    
    static boolean isAnySuperStyleNotPropagated(StyleSpecification styleSpecification, EObject obj) {
        if (styleSpecification.getSuperStyles() != null) {
            for (StyleSpecification superStyle : styleSpecification.getSuperStyles()) {
                if (isPropagated(superStyle,obj) == false) {
                    return true;
                }
            }
        }
        return false;
    }
    
    static StyleSpecification cloneAndFilterPropagatedStyles(StyleSpecification style, EObject obj) {        
        StyleSpecification clonedStyle = (StyleSpecification)EcoreUtil2.clone(style);        
        if (clonedStyle instanceof StyleSpecification) {           
            if (clonedStyle.getSuperStyles() != null) {
                EList<StyleSpecification> superStyles = clonedStyle.getSuperStyles();                                                
                List<StyleSpecification> filteredSuperStyles = new ArrayList<StyleSpecification>();                
                for (int i=0;i<superStyles.size();i++) {
                    if (isPropagated(superStyles.get(i),obj)) {
                        StyleSpecification clonedSuperStyle = cloneAndFilterPropagatedStyles(superStyles.get(i),obj);
                        filteredSuperStyles.add(clonedSuperStyle);                        
                    }                    
                }
                ((StyleSpecification) clonedStyle).getSuperStyles().clear();
                ((StyleSpecification) clonedStyle).getSuperStyles().addAll(filteredSuperStyles);                
            }                  
        }             
        return clonedStyle;
    }      

}
